@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2007 Emil Mikulic.
+ * copyright (c) 2001-2009 Emil Mikulic.
  *
  * decode.h: packet decoding.
  *
@@ -18,6 +18,7 @@
 #define NULL_HDR_LEN    4
 #define PPPOE_HDR_LEN   8
 #define SLL_HDR_LEN     16
+#define RAW_HDR_LEN     0
 
 #ifndef ETHER_ADDR_LEN
 #define ETHER_ADDR_LEN 6
@@ -25,12 +26,12 @@
 
 typedef struct {
    int linktype;
-   int hdrlen;
+   unsigned int hdrlen;
    pcap_handler handler;
 } linkhdr_t;
 
 const linkhdr_t *getlinkhdr(int linktype);
-int getcaplen(const linkhdr_t *lh);
+int getsnaplen(const linkhdr_t *lh);
 char *ip_to_str(const in_addr_t ip);
 
 typedef struct {
