@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2008 Emil Mikulic.
+ * copyright (c) 2001-2009 Emil Mikulic.
  *
  * hosts_db.c: database of hosts, ports, protocols.
  *
@@ -26,6 +26,7 @@
 #include <string.h> /* memset(), strcmp() */
 #include <unistd.h>
 
+extern int want_lastseen;
 int show_mac_addrs = 0;
 extern const char *interface;
 
@@ -273,8 +274,10 @@ format_cols_host(struct str *buf)
    str_append(buf,
       " <th><a href=\"?sort=in\">In</a></th>\n"
       " <th><a href=\"?sort=out\">Out</a></th>\n"
-      " <th><a href=\"?sort=total\">Total</a></th>\n"
-      " <th>Last seen</th>\n"
+      " <th><a href=\"?sort=total\">Total</a></th>\n");
+   if (want_lastseen) str_append(buf,
+      " <th>Last seen</th>\n");
+   str_append(buf,
       "</tr>\n");
 }
 
@@ -283,8 +286,6 @@ format_row_host(struct str *buf, const struct bucket *b,
    const char *css_class)
 {
    const char *ip = ip_to_str( b->u.host.ip );
-   struct str *lastseen = NULL;
-   time_t last_t;
 
    str_appendf(buf,
       "<tr class=\"%s\">\n"
@@ -304,26 +305,32 @@ format_row_host(struct str *buf, const struct bucket *b,
          b->u.host.mac_addr[4],
          b->u.host.mac_addr[5]);
 
-   last_t = b->u.host.last_seen;
-   if (now >= last_t)
-      lastseen = length_of_time(now - last_t);
-
    str_appendf(buf,
       " <td class=\"num\">%'qu</td>\n"
       " <td class=\"num\">%'qu</td>\n"
-      " <td class=\"num\">%'qu</td>\n"
-      " <td class=\"num\">",
+      " <td class=\"num\">%'qu</td>\n",
       b->in, b->out, b->total);
 
-   if (lastseen == NULL)
-      str_append(buf, "(clock error)");
-   else {
-      str_appendstr(buf, lastseen);
-      str_free(lastseen);
+   if (want_lastseen) {
+      time_t last_t = b->u.host.last_seen;
+      struct str *lastseen = NULL;
+
+      if (now >= last_t)
+         lastseen = length_of_time(now - last_t);
+
+      str_append(buf,
+         " <td class=\"num\">");
+      if (lastseen == NULL)
+         str_append(buf, "(clock error)");
+      else {
+         str_appendstr(buf, lastseen);
+         str_free(lastseen);
+      }
+      str_append(buf,
+         "</td>");
    }
 
    str_appendf(buf,
-       "</td>\n"
       "</tr>\n");
 
    /* Only resolve hosts "on demand" */
@@ -771,9 +778,10 @@ struct bucket *
 host_get_ip_proto(struct bucket *host, const uint8_t proto)
 {
    struct host *h = &host->u.host;
+   static const unsigned int PROTOS_MAX = 512, PROTOS_KEEP = 256;
    assert(h != NULL);
    if (h->ip_protos == NULL)
-      h->ip_protos = hashtable_make(PROTO_BITS, ports_max, ports_keep,
+      h->ip_protos = hashtable_make(PROTO_BITS, PROTOS_MAX, PROTOS_KEEP,
          hash_func_byte, free_func_simple, key_func_ip_proto,
          find_func_ip_proto, make_func_ip_proto,
          format_cols_ip_proto, format_row_ip_proto);
